library(pracma)
library(coda)
library(mcmcse)
library(dplyr)
library(rlang)
library(tidyr)
library(tibble)
library(latex2exp)
library(ggpubr)
library(LaplacesDemon)
rm(list=ls())

simstudyplot = function(MCMC_analysis_results, study ){
  
  truebeta = c()
  meanbeta = c()
  medianbeta = c()  
  truetau = c()
  meantau  = c()
  mediantau  = c()  
  truerho = c()
  meanrho = c()
  medianrho = c()
  essmcmc = c()
  for(MCMCres in MCMC_analysis_results){
    true = MCMCres$`true parameters`
    mean  = MCMCres$` posterior means`
    median  = MCMCres$` posterior medians`
    
    truebeta = c(truebeta, matrix(true[,seq(5)], ncol = 1))
    meanbeta = c(meanbeta, matrix(mean[,seq(5)], ncol = 1))
    medianbeta = c(medianbeta, matrix(median[,seq(5)], ncol = 1))
    
    truetau = c(truetau, true[,6])
    meantau = c(meantau, mean[,6])
    mediantau = c(mediantau, median[,6])
    
    truerho = c(truerho, true[,7])
    meanrho = c(meanrho, mean[,7])
    medianrho = c(medianrho, median[,7])
    essmcmc = c(essmcmc, apply(MCMCres$`ess coda`[,seq(7)], 1, min))
  }
  summary_results = as_tibble(cbind(truebeta, meanbeta, medianbeta, truetau, meantau, mediantau, truerho, meanrho, medianrho, essmcmc))
  
  
  if(study == 'study_2'){
    lpbeta = ggscatter(data = summary_results , x = 'truebeta', y = 'meanbeta',
                       xlab = expression('true '~ beta[i]),
                       ylab = expression('posterior mean '~beta[i]),
                       cor.coef = T)
     histrho = gghistogram(summary_results, x = 'meanrho', xlab = expression('posterior mean '~tau[1]), bins = 100)+
          geom_point(data =summary_results[1,] %>% add_column(zeroval = 0), aes(x = truerho, y = zeroval),colour = 'red', size = 3)
     
     
     medians_of_medians = summary_results %>%select(truetau, mediantau) %>% group_by(truetau) %>% summarise(medianmediantau = median(mediantau), .groups = 'drop_last') 
     lptau = ggscatter(summary_results, x = 'truetau', y = 'mediantau', xlab = expression('true '~tau[2]), ylab = expression('posterior median'~tau[2]))+
                geom_point(data = medians_of_medians, aes(x = truetau, y = medianmediantau), colour = 'red', size = 3)+
                geom_line(data = medians_of_medians, aes(x= truetau, y= truetau), linetype = 2)
     
     plot_ess = gghistogram(summary_results, x = 'essmcmc', xlab = 'effective sample size (coda)', bins = 100, 
                            title = paste0('median = ', round(median(summary_results$essmcmc),2), ' min = ', round(min(summary_results$essmcmc), 2)))
             
     P = ggarrange(lpbeta, histrho, lptau, plot_ess, ncol = 2, nrow = 2)
                                                                          
}else{
    if(study == 'study_1'){
      lpbeta = ggscatter(data = summary_results , x = 'truebeta', y = 'meanbeta',
                         xlab = TeX('true $\\beta_i$'),
                         ylab = TeX('posterior mean $\\beta_i$'),
                         cor.coef = T)
      lprho = ggscatter(data = summary_results , x = 'truerho', y = 'meanrho',
                         xlab = (TeX('true $\\tau_1$')),
                         ylab = TeX('posterior mean $\\tau_1$'),
                         cor.coef = T)
      
      lptau = ggscatter(data = summary_results , x = 'truetau', y = 'meantau',
                        xlab = TeX('true $\\tau_2$'),
                        ylab = TeX('posterior mean $\\tau_2$'),
                        cor.coef = T)
      
      plot_ess = gghistogram(summary_results, x = 'essmcmc', xlab = 'effective sample size (coda)', bins = 100, 
                             title = paste0('median = ', round(median(summary_results$essmcmc),2), ' min = ', round(min(summary_results$essmcmc), 2)))
      
      P = ggarrange(lpbeta, lprho, lptau, plot_ess, ncol = 2, nrow = 2)
      
      
    }}
  return(P)
  }

source('utils.R')
write =T
calculate = F
study = 'study_1'
path = paste0('../simulation_data/',study,'/')
files = dir(path)
files = files[grepl('.R', files)] # only include R files
tau_list = unique(as.numeric(gsub('.R.*','', gsub('.*=', '', files))))

if(study == 'first_study'){
  tau_list = c(-0.2,-0.1,-0.05,0,0.05,0.1,0.2,0.3,0.4,0.5)#
  files = files[apply(sapply(tau_list , function(tauval){grepl(paste0('tau=',toString(tauval)), files)}),1,any)]
}
max_sim = 1000
if(calculate){
MCMC_analysis_results = list()
for(i in seq_len(length(tau_list))){
  tau_val = tau_list[i]
  strtau = toString(tau_val)
  file    = files[which(sapply(files, function(file){grepl(paste0('tau=', toString(tau_val) ,'.R'), file)}))]
  load(paste0(path, file))
  print(paste0('loaded ', file ))
  
  posterior_means_tot = c()
  posterior_medians_tot = c()
  multiess = c()
  ess_mcmcse = c()
  ess_mcmcse_twothirds = c()
  ess_acf = c()
  abs_ss = c()
  diag_hb = c()
  ess_coda = c()
  diag_geweke = c()
  true_parameters =c()
  for(i_simres in seq(length(simresults))){
    print(i_simres)
    simresult = simresults[[i_simres]]
    if(i_simres<max_sim){
      MCMC_results = simresult$`MCMC samples`$MCMC_results
      param_interest_true = c(simresult$data$beta, simresult$data$tau, simresult$data$rho)
      if(typeof(MCMC_results)=='double' ){
        #add true parameters to list results
        trueparams = c(simresult$data$beta, simresult$data$tau, simresult$data$rho)
        true_parameters = rbind(true_parameters, trueparams )
        
        param_interest = MCMC_results[,seq(7)] #parameters of interest
        
        #add posterior means to list results
        posterior_means = apply(param_interest, 2, mean)
        posterior_means_tot = rbind(posterior_means_tot, posterior_means )
        
        #add posterior medians to list results
        posterior_medians = apply(param_interest, 2, median)
        posterior_medians_tot = rbind(posterior_medians_tot, posterior_medians )
       #adding effective sample size calculated using acf implementation to result
        ess_coda = rbind(ess_coda,  tryCatch({effectiveSize(param_interest)}, error= function(dummy){c()}))
        
        # #adding absolute sample size
        abs_ss = rbind(abs_ss, dim(param_interest)[[1]])
        
           }}
    MCMC_analysis_results[[strtau]][[' posterior means']] = posterior_means_tot
    MCMC_analysis_results[[strtau]][['true parameters']] = true_parameters
    MCMC_analysis_results[[strtau]][[' posterior medians']] = posterior_medians_tot
       MCMC_analysis_results[[strtau]][['ess coda']] = ess_coda
    MCMC_analysis_results[[strtau]][['abs ss']] = abs_ss
  }}
# 
if(write){
save(MCMC_analysis_results, file = paste0('MC_analysis_results_', study,'.Rds'))}
}else{
  load( paste0('MC_analysis_results_', study,'.Rds'))
}

P = simstudyplot(MCMC_analysis_results, study)
ggsave(paste0('../figures/summary_simstudy_', study, '.png'), P, width = 10, height = 10)



