library(pracma)
library(robustX)
library(Hmisc)
library(jocre)
library(bfp)
library(xtable)
library(tables)
rm(list=ls())

source('utils.R')

study = 'study_1'
path = paste0('../simulation_data/',study,'/')
files = dir(path)
files = files[grepl('.Rdata', files)] # only include R files


tau_list = unique(as.numeric(gsub('.R.*','', gsub('.*=', '', files))))
write_results = T


med_results_list = c()
avg_results = c()
list_all_errors = list()

coverage_tau = c()
coverage_rho = c()
coverage_beta = c()
alpha_range = c(0.6,0.7,0.8,0.9,0.95,0.99)
ESE = c()
for(i in seq_len(length(tau_list))){
  tau_val = tau_list[i]
  file    = files[which(sapply(files, function(file){grepl(paste0('tau=', toString(tau_val) ,'.R'), file)}))]
  print(tau_val)
  load(paste0(path, file))
  print(paste0('loaded ', file ))
  error_results = c()
  partial_res_covg_tau = c()
  partial_res_covg_rho = c()
  partial_res_covg_beta = c()
  
  
  p=5
  L = 0
  for(simresult in simresults){
    if(L<1000){
    MCMC_results = simresult$`MCMC samples`$MCMC_results
    if(typeof(MCMC_results)=='double' ){
      L = L+1
    data       = simresult$data
  
    beta = data$beta
    tau = data$tau
    rho = data$rho
    
    beta_MCMC = MCMC_results[ , grepl( "beta" , colnames( MCMC_results ) ) ]
    gamma_MCMC = MCMC_results[ , grepl( "gamma" , colnames( MCMC_results ) ) ]
    tau_MCMC  = MCMC_results[, 'tau_hat']
    rho_MCMC  = MCMC_results[, 'rho_hat']
    
    beta_pointest = apply(beta_MCMC,2,median)
    tau_pointest = median(tau_MCMC)
    rho_pointest = median(rho_MCMC)
  
    relerr_PE_beta = mean((beta_pointest - beta)/abs(beta))
    relerr_PE_rho = (rho_pointest - rho)/abs(rho)
    
    
    RP_beta = mean(apply(beta_MCMC,2,std)/abs(beta))
    if(grepl('study_2', study)){
      SD_tau = std(trim_data(tau_MCMC))
    }else{
      SD_tau = std(tau_MCMC)
    }
    
    RP_rho = std(rho_MCMC)/abs(rho)

    HPD_beta = apply(beta_MCMC, 2, function(v){empiricalHpd(v, 0.95)})
    HPD_tau = empiricalHpd(tau_MCMC,0.95)
    HPD_rho = empiricalHpd(rho_MCMC,0.95)
    
    cvg95_beta = mean((beta<HPD_beta[2,])&(beta>HPD_beta[1,]))
    cvg95_tau = (tau<HPD_tau[2])&(tau>HPD_tau[1])
    cvg95_rho = (rho<HPD_rho[2])&(rho>HPD_rho[1])
    
    
    for(i in seq(p)){
    ci_beta  = sapply(alpha_range, function(alpha){empiricalHpd(beta_MCMC[,i], alpha)})
    covg_beta    = (beta[i]>ci_beta[1,])&(beta[i]<ci_beta[2,])
    partial_res_covg_beta = rbind(partial_res_covg_beta, covg_beta)
    }
    ci_tau  = sapply(alpha_range, function(alpha){empiricalHpd(tau_MCMC, alpha)})
    
    ci_rho  = sapply(alpha_range, function(alpha){empiricalHpd(rho_MCMC, alpha)})
  
    covg_rho    = (rho>ci_rho[1,])&(rho<ci_rho[2,])
    covg_tau    = (tau>ci_tau[1,])&(tau<ci_tau[2,])
  
    partial_res_covg_rho = rbind(partial_res_covg_rho ,covg_rho)
    partial_res_covg_tau = rbind(partial_res_covg_tau ,covg_tau)
    error_results = rbind(error_results, c(relerr_PE_beta, tau_pointest, relerr_PE_rho,
                                           RP_beta, SD_tau, RP_rho, 
                                           cvg95_beta, cvg95_tau, cvg95_rho))
    }else{
  # print(simresult$error)
      1

}}}
  colnames(error_results) = c('RB \beta', 'median \tau', 'RB \rho',
                              'RP \beta', 'SD \tau','RP \rho',
                              'cvg95 \beta', 'cvg95 \tau', 'cbg95 \rho')
  
  coverage_beta = rbind(coverage_beta, apply(partial_res_covg_beta,2,mean))
  coverage_rho= rbind(coverage_rho, apply(partial_res_covg_rho,2,mean))
  coverage_tau = rbind(coverage_tau, apply(partial_res_covg_tau,2,mean))
  
  median_res = append(apply(error_results[,seq(6)], 2, median), round(apply(error_results[,seq(7,9)], 2, mean),3))
  median_res[c(1,3,4,5,6)] = round(median_res[c(1,3,4,5,6)],3)
  
  median_res[2] = round(median_res[2],3)
  ESE = c(ESE , std(error_results[,2]))
  list_all_errors = append(list_all_errors, list(error_results))
  med_results_list = c(med_results_list, median_res)
  print(L)
}

#obtaining parameter recovery table
paramnames   = c("$\\beta$", "$\\tau$", "$\\rho$")
num_tau_vals = length(tau_list)
colFactor1 = rep(c('RB','median','RB','CV','SD','CV','CP95','CP95','CP95'), num_tau_vals)
colFactor2 = rep(rep(paramnames, 3), num_tau_vals)
rowFactor  = rep(sapply(tau_list, function(val){round(val,4)}), each = 9)
results_df <- data.frame(rowFactor1 = rowFactor,
                         colFactor1 = colFactor1,
                         colFactor2 = colFactor2,
                         x = med_results_list)

tau = rep(sapply(tau_list, function(val){round(val,4)}), each = 9)
results_table <- tabular(RowFactor(tau, name = '$\\tau_2$') ~
                                   Heading()*Factor(colFactor2,
                                                    levelnames =  c("$\\bm{\\hat{\\beta}}$",
                                                                    "$\\hat{\\tau}_1$",
                                                                    "$\\hat{\\tau}_2$" ))*
                                   Heading()*Factor(colFactor1,
                                                    levelnames = c('CP95', 'CV', 'median', 'RB','SD'))*Heading()*(x)*Heading()*(mean)*
                           DropEmpty(empty=""),
                                  data = results_df)

results_table
if(write_results){
  write(tables::latex.tabular(results_table)[[1]], file = paste0('../tables/paramrecov_',study,'.tex'))
}

#obtaining coverage table
covg_res  = round(c(apply(coverage_beta,2,mean), apply(coverage_tau,2,mean), apply(coverage_rho,2,mean) ),2)
colFactor = c('0.6','0.7','0.8','0.9','0.95','0.99')
rowFactor = rep(c('$\\beta$','$\\tau_2$','$\\tau_1$'),each = length(colFactor))
covg_df = data.frame(rowFactor=rowFactor, colFactor=colFactor, x = covg_res)
covg_table <- tabular(RowFactor(rowFactor, name = 'param') ~
                           Heading()*Factor(colFactor, levelnames = c('0.6','0.7','0.8','0.9','0.95','0.99'))*Heading()*(x)*Heading()*(mean)*DropEmpty(empty = ''),
                         data = covg_df)
covg_table

if(write_results){
  write(tables::latex.tabular(covg_table)[[1]], file = paste0('../tables/coverage_table_',study,'.tex'))
}


