
generate_data = function(K, n, group_sizes , tau_true, rho_true, alpha_true, p, beta_true = NULL, p_cens = 0.01, meas_times = NULL,  X = NULL){
  # generate data according to the LT-BCSM, K is the number of observations per patient, n is the number of treatment groups, p is the number of covariates
  # p_cens is the percentage of left/right censored observations, meas_times are the unique observed values of L,R, X is the covariate matrix
  
  num_Z_values       = sum(group_sizes)*K # length of total Z vector
  sumGS              = sum(group_sizes) 
  
  #define covariates and regression coefs
  if(p==0){
    beta_true = NULL
    X = NULL
  }else{
    if(is.null(beta_true)){beta_true   = runif(p,-1,1)} # if beta is not specified, sample uniformly on [-1,1]
    if(is.null(X)){X = kron(cbind(matrix(rnorm((p-round(p/2))*sumGS), ncol = p-round(p/2)), matrix(as.numeric(runif(sumGS*round(p/2))<0.5), ncol = round(p/2))), ones(K,1))} # if covariates unspecified, sample half the covariates standard normal and the rest bernoulli 1/2
  }
  
  #sample Z
  Z_vec = c()
  cuGS = append(0, cumsum(group_sizes)*K)# cumulative group sizes
  for(i in seq(n)){
    m         = group_sizes[i]
    i_cluster = seq(cuGS[i]+1, cuGS[i+1])
    sqrtmat = diag(m*K) + 1/(m*K)*(sqrt(1+K*rho_true+m*K*tau_true)-sqrt(1+K*rho_true)) + 1/K*(sqrt(1+K*rho_true)-1)*kron(diag(m),ones(K,K))# cholesky of Sigma
    
    if(is.null(X)){
      Z_vec = append(Z_vec, sqrtmat%*%rnorm(m*K))
    }else{
      Z_vec = append(Z_vec, sqrtmat%*%rnorm(m*K) + X[i_cluster,]%*%beta_true)
    }
  }
  
  #determine L,R
  if(is.null(meas_times)){meas_times = c(seq(0,30, 0.5), Inf)}
  alpha_T_i = alpha_true(meas_times)
  L_i = sapply(Z_vec, function(z){max(which(alpha_T_i<=z))})
  L = meas_times[L_i]
  R = meas_times[L_i+1]
  R[is.na(R)]=Inf
  
  #random left censoring
  indc_not_cens        = (L != 0) & (R<Inf)
  num_not_cens         = sum(indc_not_cens)
  idx_new_left_cens    = sample(sum(num_not_cens), round(p_cens/2*num_not_cens))
  L[indc_not_cens][idx_new_left_cens] = 0
  
  #random right censoring
  indc_not_cens         = (L != 0) & (R<Inf)
  num_not_cens          = sum(indc_not_cens)
  idx_new_right_cens    = sample(sum(num_not_cens), round(p_cens/2*num_not_cens))
  R[indc_not_cens][idx_new_right_cens] = Inf
  
  results = list(L,R,X,group_sizes, beta_true, alpha_true, tau_true, rho_true, Z_vec,K)
  names(results) = c('L','R','X','group_sizes','beta','alpha','tau','rho','Z','K')
  return(results)
}


generate_data_stent  = function(tau_true, rho_true, group_sizes, K, p=5){
  n=length(group_sizes)
  deg    = 4
  coefs = c(-2.260342972,  0.090799332, -0.006138751,  0.041676993,0.056171570
            ,0.013506954
            ,  0.052819925, -0.006168368
            , 0.046775983
            , -0.001841741
            ,  0.026547578
            , -0.009338771
            ,  0.018170813
            ,  0.001358289
            ,  0.012681123
            , 0.014953817
            , 0.056200515
            , -0.018658497
            ,  0.035219972
            , 0.005706857
            ,  0.019104873
            ,  0.002860829
            , -0.004298770
            ,  0.002000133)
  knots_bl = c(1.00000 , 19.84211 , 38.68421  ,57.52632,  76.36842,  95.21053, 114.05263 ,132.89474 ,151.73684, 170.57895, 189.42105, 208.26316, 227.10526, 245.94737, 264.78947, 283.63158 ,302.47368 ,321.31579 ,340.15789 ,359.00000)
  alpha_true  = get_spline_function(knots_bl, deg, coefs)
  
  # times = seq(0, 359)
  # baseline_plot = ggplot() + geom_line(aes(x = times, y = pnorm(alpha_true(times))), size = 1.2) +
  #   xlab('Time (days)') + ylab('marginal incidence rate for any of the events') +
  #   scale_y_continuous(labels = scales::percent) +
  #   scale_x_continuous(breaks = seq(0,360, 30))+ theme_bw()
  # ggsave(baseline_plot, file = paste0(results_path, fig_path, 'baseline_plot_secondstudy.pdf'), device = cairo_pdf, width = 180, height = 110, units = 'mm')
  
  
  #define covariates and regression coefs
  sumGS = sum(group_sizes)
  if(p==0){
    beta_true = c(0)
    X = NULL
  }else{
      beta_true   = runif(p,-1,1)
      X           = kron(cbind(matrix(rnorm((p-round(p/2))*sumGS), ncol = p-round(p/2)), matrix(as.numeric(runif(sumGS*round(p/2))<0.5), ncol = round(p/2))), ones(K,1)) # covariates
  
  }

  #sample Z
  Z_vec = c()
  cuGS = append(0, cumsum(group_sizes)*K)
  for(i in seq(n)){
    m         = group_sizes[i]
    i_cluster = seq(cuGS[i]+1, cuGS[i+1])
    
    sqrtmat = diag(m*K) + 1/(m*K)*(sqrt(1+K*rho_true+m*K*tau_true)-sqrt(1+K*rho_true)) + 1/K*(sqrt(1+K*rho_true)-1)*kron(diag(m),ones(K,K))
    if(p>0){
      Z_vec = append(Z_vec, sqrtmat%*%rnorm(m*K) + X[i_cluster,]%*%beta_true)
    }else{
      Z_vec = append(Z_vec, sqrtmat%*%rnorm(m*K) )
    }
    # assertthat::assert_that(norm(diag(m*K) + tau_true + rho_true*kron(diag(m), ones(K,K)) - sqrtmat%*%sqrtmat)<1e-10)
    # Z_vec     = append(Z_vec, rmvnorm(1, mean = X[i_cluster,]%*%beta_true, sigma = diag(m*K) + tau_true + rho_true*kron(diag(m), ones(K,K))))
  }
  
  #determine L,R
  meas_times  = c(seq(0, 359), Inf)
  alpha_T_i = alpha_true(meas_times)
  L_i = sapply(Z_vec, function(z){max(which(alpha_T_i<=z))})
  L = meas_times[L_i]
  R = meas_times[L_i+1]
  results = list(L,R,X,group_sizes, beta_true, alpha_true, tau_true, rho_true, Z_vec,K)
  names(results) = c('L','R','X','group_sizes','beta','alpha','tau','rho','Z','K')
  return(results)
}

if (sys.nframe() == 0){ # if name == main
  
  data = generate_data_stent(0, 0.4, c(1173,1169,1172), 3,p=0)
  
  # obtaining spline matrix
  if(is.null(data$X)){p=0}else{p=dim(data$X)[[2]]}
  L = data$L
  R = data$R
  LRunqpair = t(as.data.frame(c(L[1],R[1])))
  for(i in seq(2,length(L))){
    if(length(intersect(which(LRunqpair[,1] == L[i]), which(LRunqpair[,2] == R[i]))) == 0){
      LRunqpair = rbind(LRunqpair, c(L[i], R[i]))
    }
  }
  

  
  estcdfval = pnorm(data$alpha(LRunqpair[,2])/sqrt(1+data$tau+data$rho)) - pnorm(data$alpha(LRunqpair[,1])/sqrt(1+data$tau+data$rho))
  empiricalcdf = sapply(seq(dim(LRunqpair)[1]), function(i){mean(data$L >= LRunqpair[i,1] & data$R <= LRunqpair[i,2])})
  
  
  plot(empiricalcdf, estcdfval, xlim = c(0.95,1), ylim = c(0.95,1))
  lines(empiricalcdf, empiricalcdf, type = 'l', col = 'black')
  
  }







