
link_function = function(gamma, L_splines,R_splines,X=NULL,beta=NULL){
  if(any(c(is.null(beta), is.null(X)))){
    #get log likelihood ofbeta and gamma lin wang model
    negll = -sum(log(pnorm(R_splines%*%gamma ) - pnorm(L_splines%*%gamma  )))
    if(negll == Inf){
      negll = 1e10
    }
  }else{
    #get log likelihood ofbeta and gamma lin wang model
    negll = -sum(log(pnorm(R_splines%*%gamma - X%*%beta) - pnorm(L_splines%*%gamma - X%*%beta )))
    if(negll == Inf){
      negll = 1e10
    }}
  return(negll)
}


get_MLE = function( L,R,X,knots, deg){
  #obtain Max likelihood estimate of the parameters under zero covariance
  
  indc_full_cens = L == 0 & R == Inf
  L = L[!indc_full_cens]
  R = R[!indc_full_cens]
  
  if(!is.null(X)){p = dim(X)[2]
  X = X[!indc_full_cens, ]}
  
  L_splines          = get_spline_matrix_points(L,knots, deg)
  R_splines          = get_spline_matrix_points(R, knots, deg)

  num_funcs = dim(L_splines)[2]-1

  if(is.null(X)){
      log_likelihood_params = function(x){link_function( x[seq(1,num_funcs+1)], L_splines,R_splines)}
      optres = constrOptim(theta=c( 0, 0.001*ones(1,num_funcs)), grad = NULL, f=log_likelihood_params, ui = bdiag(0, eye(num_funcs)),
                           ci = c(-1, zeros(1,num_funcs)), outer.eps = 0.005)
      soln = optres$par
      
    }else{
      log_likelihood_params = function(x){link_function(x[seq(p+1,p+num_funcs+1)], L_splines,R_splines,X=X, beta = x[seq(p)])}
      optres = constrOptim(theta=c(ones(1,p+1), 0.001*ones(1,num_funcs)), grad = NULL, f=log_likelihood_params, ui = bdiag(zeros(p+1), eye(num_funcs)),
                           ci = c(-ones(1,p+1), zeros(1,num_funcs)), outer.eps = 0.005)
      soln = optres$par
    }
  return(soln)
}

if(sys.nframe() == 0){
  source('utils.R')
  source("generate data.R")
  library(pracma)
  library(mvtnorm)
  library(splines2)
  
  #setting constants
  K              = 5     # number of events
  n              = 300   # number of groups
  Egs            = 5     # expected group size
  max_gs         = 10
  group_sizes    = sapply(rpois(n, Egs), function(x){min(max_gs, max(x,2))})
  deg            = 4                      # degree of the I-splines
  p              = 5                      # num covariates
  rho_true       = runif(1, -1/K , 0.3)   # covariance events per subject
  tau_true       = runif(1, -1/(K*max(group_sizes)) - rho_true/max(group_sizes), 0.3)
  knots_bl       = seq(0,7,0.5)
  coefs          = c(-6, runif(length(knots_bl)+deg-1, 0, 2))
  coefs[seq(2,length(coefs))] = coefs[seq(2,length(coefs))]/sum(coefs[seq(2,length(coefs))] )*12
  alpha_true     = get_spline_function(knots_bl, deg, coefs)
  
  #generate data, whole numbers would mean that the endpoints are integers
  data          = generate_data(K, n, group_sizes , tau_true, rho_true, alpha_true, p)
  
  L           = data[[1]]        # left interval endpoints
  R           = data[[2]]        # right interval endpoints
  X           = data[[3]]        # covariate values
  group_sizes = data[[4]]
  beta_true   = data[[5]]
  alpha_true  = data[[6]]
  
  # determining knots, which are taken to be equidistant (distance dknot) between 0 and the maximal
  # finite endpoint + dknot
  times       = sort(unique(append(L,R)))
  meas_times  = times[abs(times)<Inf & times >0]
  knots       = seq(min(meas_times)*0.99, max(meas_times)*1.01, length.out = 20)
  
  res = get_MLE( L,R,X,knots, deg)
  params = res[seq(p)]
  gamma_opt = res[-seq(p)]
  print(params)
  print(c(beta_true))
  S           = get_spline_matrix_points(meas_times, knots = knots, deg = deg)
  plot(meas_times, S%*%gamma_opt, type = 'l')
  lines(meas_times, alpha_true(meas_times), col = 'red')
}