library(extraDistr)
sample_gamma = function(gamma_hat, Z_hat, sigma, X, beta_hat,  num_funcs, IndcDeltaNonzero, clustercoef_matrix, sum_sigma_clusters, cluster_indc_mat,
                        subject_coef, sum_sigma_subjects, subject_indc_mat, IndcIntvCens, delta_left_noncens, eta_hat, m_0, v_0, M = Inf){
  # sample spline coefficients for a linear transormation model with monotone splines and covariance matrix coming from a 2-way nested design.
  
  # determining matrices used for regression 
  precision_gamma = t(sigma)%*%sigma + t(sum_sigma_clusters)%*%clustercoef_matrix%*%sum_sigma_clusters + subject_coef*t(sum_sigma_subjects)%*%sum_sigma_subjects
  precision_gamma[1,1] = precision_gamma[1,1] + v_0
  sd_gamma = 1/sqrt(diag(precision_gamma))

  IPmat = t(sigma) + t(sum_sigma_clusters)%*%clustercoef_matrix%*%t(cluster_indc_mat) + subject_coef*t(sum_sigma_subjects)%*%t(subject_indc_mat)
  Z_hat_normalized = IPmat%*%(Z_hat - X%*%beta_hat)
  sigma_normalized = IPmat%*%sigma

  for(i in seq(1, num_funcs+1)){
    cond_sd         = sd_gamma[i]

    if(cond_sd<1e4){  # if there are almost no observations in a number of classes, cond_sd is Inf (or large) and it is better to set gamma_i=0
      gamma_hat[i]       = 0                              # excluding gamma_i from multiplication is done by setting it to zero
      IndcDeltaNonzero_i = IndcDeltaNonzero[,i]           # indicators for patients where delta_left is nonzero

      #sampling gamma_i
      prior_mean         = if(i>1){eta_hat}else{-v_0*m_0}
      mean               = as.numeric((cond_sd^2)*(Z_hat_normalized[i] - sigma_normalized[i,]%*%gamma_hat - prior_mean))

      if(i==1){
        gamma_hat[i]       = rtnorm(1,a = -M, b = M, mean = mean, sd =cond_sd)
      }else{
        # setting the lower truncation boundary for truncated normally distributed gamma, the delta's have to be corresponding
        # to the patient that don't have left/right censoring, and delta_i should be nonzero as well (because we divide by it).
        thr_gamma          = max(append(1e-14, (Z_hat[IndcIntvCens][IndcDeltaNonzero_i] - delta_left_noncens[IndcDeltaNonzero_i,]%*%gamma_hat)/
                                            delta_left_noncens[IndcDeltaNonzero_i,i]))

	
        gamma_hat[i]      = rtnorm(1, a = thr_gamma, mean = mean, sd = cond_sd)
      }

    }else{
      gamma_hat[i] = if(i>1){rexp(1,eta_hat)}else{rnorm(1,m_0,1/sqrt(v_0))}}
  }
  return(gamma_hat)
}

if (sys.nframe() == 0){ # if name == main
  library('pracma')
  library(splines2)
  library(Matrix)
  source("generate data.R")
  source("utils.R")
  
  #setting constants
  K              = 5    # number of events
  n              = 100    # number of groups
  Egs            = 5
  deg            = 4      # degree of the I-splines
  p              = 5
  maxgs          = 10
  
  group_sizes    = sapply(rpois(n, Egs), function(x){min(maxgs, max(x,2))})
  rho_true       = 0.1  # covariance events per subject
  tau_true       = 0.1
  knots_bl       = seq(0,7,0.5)
  coefs          = c(-6, runif(length(knots_bl)+deg-1, 0, 2))
  coefs[seq(2,length(coefs))] = coefs[seq(2,length(coefs))]/sum(coefs[seq(2,length(coefs))] )*15
  alpha_true     = get_spline_function(knots_bl, deg, coefs)
  
  #generate data, whole numbers would mean that the endpoints are integers
  sumGS              = sum(group_sizes)
  X = cbind(matrix(rnorm((p-round(p/2))*sumGS*K), ncol = p-round(p/2)), matrix(as.numeric(runif(sumGS*round(p/2)*K)<0.5), ncol = round(p/2)))
  data        = generate_data(K, n, group_sizes , tau_true, rho_true, alpha_true, p=p, X=X)
  L           = data$L        # left interval endpoints
  R           = data$R        # right interval endpoints
  X           = data$X        # covariate values
  group_sizes = data$group_sizes
  beta_true   = data$beta
  rho_true    = data$rho
  tau_true    = data$tau
  Z_hat      = data$Z
  if(is.null(X)){p=0}else{p=dim(X)[2]}
  
  #determine L,R
  meas_times = seq(0.1,30, 0.1)
  knots       = seq(min(meas_times)*0.99, max(meas_times)*1.01, length.out = 20)
  S = get_spline_matrix_points(meas_times, knots = knots, deg = deg)
  gamma_true = pinv(t(S)%*%S)%*%t(S)%*%alpha_true(meas_times)
  gamma_true = c(gamma_true[1], sapply(gamma_true[-1], function(x){max(x,1e-10)}))
  
  meas_times_ext = c(0, meas_times, Inf)
  alpha_T_i = c(-Inf, S%*%gamma_true, Inf)
  L_i = sapply(Z_hat, function(z){max(which(alpha_T_i<=z))})
  L = meas_times_ext[L_i]
  R = meas_times_ext[L_i+1]
  R[is.na(R)]=Inf
  
  p_cens = 0.1
  #random left censoring
  indc_not_cens        = (L != 0) & (R<Inf)
  num_not_cens         = sum(indc_not_cens)
  idx_new_left_cens    = sample(sum(num_not_cens), round(p_cens/2*num_not_cens))
  L[indc_not_cens][idx_new_left_cens] = 0
  
  #random right censoring
  indc_not_cens         = (L != 0) & (R<Inf)
  num_not_cens          = sum(indc_not_cens)
  idx_new_right_cens    = sample(sum(num_not_cens), round(p_cens/2*num_not_cens))
  R[indc_not_cens][idx_new_right_cens] = Inf
  
  L_splines          = get_spline_matrix_points(L,knots, deg)
  R_splines          = get_spline_matrix_points(R, knots, deg)
  
  #determine measurement times (regular event-time interval endpoints) and spline knots
  times       = sort(unique(append(L,R)))
  meas_times  = times[abs(times)<Inf & times >0]
  S = get_spline_matrix_points(meas_times, knots = knots, deg = deg)
  
 
  # obtaining matrix sigma (such that sigma%*%gamma = alpha(t_i) from Lin Wang article),
  T_LW               = L
  T_LW[T_LW == 0]    = R[T_LW == 0]
  sigma              = get_spline_matrix_points(T_LW, knots = knots, deg = deg)
  

  Z_hat = sigma%*%gamma_true - Z_hat                                 # Z in lin wang article was mirrored and translated by sigma%*%gamma, so we have to correct.
  
  if(is.null(X)){X = 0
  beta_true = 0}
  X = -X

  # obtaining matrices delta_left and delta_right such that delta_left%*%gamma = -alpha(R) + alpha(t_i).
  delta_left         = -R_splines + sigma
  delta_right        = -L_splines + sigma
  num_funcs          = dim(delta_left)[2] - 1
  
  #calculating matrices with which to transform the data later on
  cugs                  = c(0, cumsum(group_sizes))
  cluster_indc_mat      = bdiag(lapply(group_sizes, function(gs){ones(gs*K,1)}))
  subject_indc_mat      = bdiag(lapply(seq(sum(group_sizes)), function(i){ones(K,1)}))
  sum_sigma_clusters    = t(cluster_indc_mat)%*%sigma   
  sum_sigma_subjects    = t(subject_indc_mat)%*%sigma
  
  #obtaining delta left where there is no censoring, and furthermore indicators when this is nonzero (used in gamma sampling)
  IndcIntvCens       = L>0&is.finite(R)           
  delta_left_noncens = delta_left[IndcIntvCens,]             
  IndcDeltaNonzero   = delta_left_noncens < 0
  
  subject_coef       = -rho_true/(K*rho_true + 1)
  clustercoef_matrix = Diagmat(-tau_true/((K*rho_true + 1)*(1+K*(rho_true + group_sizes*tau_true))))

  gamma_hat = gamma_true ## NOTE: we have to start with a feasible gamma, otherwise we get weird results, in the Gibbs sampler we adjust Z to gamma, but we cannot do that here
  eta_hat   = 1
  m_0       = 0
  v_0       = 0
  M  =1000
  a_eta = 0
  b_eta = 0

  niter = 1000
  burnin = 0
  MCMCres = c()
  for(i in seq(niter)){
    print(i)
    gamma_hat = sample_gamma(gamma_hat, Z_hat, sigma, X, beta_true,  num_funcs, IndcDeltaNonzero, clustercoef_matrix, sum_sigma_clusters, cluster_indc_mat,
                             subject_coef, sum_sigma_subjects, subject_indc_mat, IndcIntvCens, delta_left_noncens, eta_hat, m_0, v_0, M=M )
    p_ub = pgamma(M, a_eta + num_funcs, b_eta + sum(gamma_hat[seq(2, num_funcs+1)]))
    eta_hat = qgamma(runif(1)*p_ub, a_eta + num_funcs, b_eta + sum(gamma_hat[seq(2, num_funcs+1)]))
    print(gamma_hat)
    MCMCres = rbind(MCMCres, t(gamma_hat))
  }

  #obtaining baseline estimate LW implementation
  median_gamma = apply(MCMCres[seq(burnin,niter),], 2, median)
  median_baseline= S%*%median_gamma

  #plotting baselines
  plot(meas_times, median_baseline, xlab = 'T', ylab = expression(alpha_true), lwd = 2, type = 'l',
       ylim = c(min(alpha_true(meas_times[1]), median_baseline[1]), max(alpha_true(meas_times[length(meas_times)]),
                                                                        median_baseline[length(meas_times)])))

  lines(meas_times, alpha_true(meas_times), col = 'red', lwd = 2)
  legend(min(meas_times),alpha_true(max(meas_times)),  c('median sampled baseline', 'true baseline'), lty = c(1,1), col = c('black', 'red'))

  }


