
package_list = c('pracma','splines2','Matrix','doParallel', 'extraDistr', 'invgamma',
               'tmvtnorm','mvtnorm', 'assertthat')
for(package_name in package_list){

  do.call(library, list(package = package_name, character.only = TRUE))}

rm(list=ls()) # remove previous globals
source("generate data.R")
source("Gibbs_sampler.R")
source("parameter sampling/sample_Z.R")
source("parameter sampling/sample_covariances.R")
source("parameter sampling/sample_beta.R")
source("parameter sampling/sample_gamma.R")
source('simulate_for_study_1.R')
source("utils.R")
source('get_MLE.R')

#setting constants
K              = 5     # number of events
n              = 50    # number of groups
Egs            = 5
max_group_size = 10
niter          = 6000     # length of Markov chain
burnin         = 3000     # burn-in period Markov chain
deg            = 4        # degree of the I-splines
p              = 5
tau_list       = c(-0.2,-0.1,-0.05,0,0.05,0.1,0.2,0.3,0.4,0.5)
nsim           = rep(1000,length(tau_list))
neff           = 100
min_rhos        = -1/K - max_group_size*tau_list + 1e-10
len_sim_batch  = 64*2
check_convg = T


#simulate the trials in parallel, storing the result after a batch is completed. The results are denoted with "part"
# the path to the correct folder for storage might have to be changed
num_sim_batch = ceil(nsim/len_sim_batch)
 cores <- detectCores(logical = FALSE)
 cl <- makeCluster(cores)
 registerDoParallel(cl, cores=cores)
for(i in seq(length(tau_list))){
  tau_true = tau_list[i]
   rem_num_sim   = nsim[i]# remaining number of simulations
  for(j in seq(num_sim_batch[i])){
     to_sim = min(rem_num_sim, len_sim_batch)
       simresults = foreach(icount(to_sim), .combine = append, .packages = c('extraDistr','pracma', 'mvtnorm','Matrix', 'splines2','invgamma', 'assertthat',
                                                                           'tmvtnorm','mvtnorm', 'coda')) %dopar% {
       list(simulate_for_study_1(K,n,p,deg,niter, neff, tau_true = tau_true, Egs = Egs,check_convg = check_convg, burnin = burnin, sample_effective = TRUE, max_group_size=max_group_size, min_rho = min_rhos[i]))
      }
   
     rem_num_sim = rem_num_sim - to_sim
     save('simresults', file = paste('./simdata/study_1/partial_data/',toString( Sys.Date()),'_tau=',toString(tau_true),'_part_',toString(j),'.rds', sep = ''))
   }
  }
stopCluster(cl)
