
package_list = c('pracma','splines2','Matrix','doParallel', 'extraDistr', 'invgamma',
                 'tmvtnorm','mvtnorm', 'assertthat', 'coda')
for(package_name in package_list){
  do.call(library, list(package = package_name, character.only = TRUE))}

rm(list=ls()) # remove previous globals
source("generate data.R")
source("Gibbs_sampler_study_2.R")
source("parameter sampling/sample_Z.R")
source("parameter sampling/sample_covariances.R")
source("parameter sampling/sample_beta.R")
source("parameter sampling/sample_gamma.R")
source('simulate_for_study_2.R')
source("utils.R")
source('get_MLE.R')

#setting constants
niter          = 6000# length of Markov chain
burnin         = 3000# burn-in period Markov chain
deg            = 4      # degree of the I-splines
neff           = 100
tau_list       = c(0, 0.025, 0.05, 0.075, 0.1,0.2,0.3)
nsim           = rep(1000, length(tau_list))
group_sizes    = c(1169 ,1173, 1172 )# 
max_group_size = max(group_sizes)
rho_true       = 0.6
K              = 3
len_sim_batch  = 64*2
test_convg = T
sample_effective = T
print_ = F


#simulate the trials in parallel, storing the result after a batch is completed. The results are denoted with "part"
# the path to the correct folder for storage might have to be changed
num_sim_batch = ceil(nsim/len_sim_batch)
cores <- detectCores(logical = FALSE)
cl <- makeCluster(cores)
registerDoParallel(cl, cores=cores)
for(i in 1:length(tau_list)){
   tau_true = tau_list[i]
  rem_num_sim   = nsim[i]# remaining number of simulations
for(j in seq(num_sim_batch[i])){
  to_sim = min(rem_num_sim, len_sim_batch)
  simresults = foreach(icount(to_sim), .combine = append, .packages = c('extraDistr','pracma', 'mvtnorm','tmvtnorm','Matrix', 'splines2','invgamma', 'assertthat', 'coda')) %dopar% {
    list(simulate_for_study_2(tau_true, rho_true, group_sizes, deg,burnin, niter, check_convg = test_convg, sample_effective = sample_effective, neff, print_ = print_))
  }
  rem_num_sim = rem_num_sim - to_sim
  save('simresults', file = paste('./simdata/study_2/partial_data/simresults_',toString( Sys.Date()),'_realistic_tau=',toString(tau_true),'_part_',toString(j),'.rds', sep = ''))
}
}
stopCluster(cl)
