  
  simulate_for_study_1 = function(K, n, p, deg, niter, neff, Egs = NULL, group_sizes=NULL, max_group_size = Inf, tau_true=NULL, rho_true=NULL, 
                            alpha_true=NULL, burnin=0,  sample_effective = TRUE,check_convg = TRUE, p_cens = 0.01, min_rho = -Inf, print_progress = FALSE){
    
    if(is.null(group_sizes)){
      group_sizes = sapply(rpois(n, Egs), function(x){min(max(2,x), max_group_size)})
    }
    if(is.null(rho_true)){
      rho_true       = runif(1, max(min_rho, -1/K) , max(0.5, min_rho+0.5))   # covariance events per subject   
    }  
    if(is.null(tau_true)){
      tau_true       = runif(1, max(-1/(K*group_sizes) - rho_true/group_sizes), 0.5)
    }
    if(is.null(alpha_true)){
      alpha_true     = log
    }
    
    #generate data, whole numbers would mean that the endpoints are integers
    data          = generate_data(K, n, group_sizes , tau_true, rho_true, alpha_true, p)
    
    L           = data[[1]]        # left interval endpoints
    R           = data[[2]]        # right interval endpoints
    X           = data[[3]]        # covariate values
    group_sizes = data[[4]]
    beta_true   = data[[5]]
    alpha_true  = data[[6]]
    
    #determine measurement times (regular event-time interval endpoints) and spline knots
    times       = sort(unique(append(L,R)))
    meas_times  = times[abs(times)<Inf & times >0]
    knots       = seq(min(meas_times)*0.99, max(meas_times)*1.01, length.out = 10)
    
    #try to find the parameters
    results = tryCatch(
      {#perform initialization procedure
      MLE = get_MLE(L,R,X,knots, deg)

      # obtain MH output
      seed = round(runif(1)*10^8)
      seed_info = list(seed = seed)
      MH_samples = Gibbs_sampler(K, L, R, group_sizes, knots, deg, niter, neff, seed_info=seed_info, X=X, burnin = burnin ,
                             sample_effective = sample_effective, init_mean = MLE, print_progress = F,
                             true_params = list(beta_true = beta_true, tau_true = tau_true, rho_true = rho_true), a_rho = 0.001, b_rho = 0.001,
                                                a_tau = 0.001, b_tau = 0.001, M = 1000)
      results = list(MH_samples, data, knots, deg, seed_info)
      names(results) = c("MCMC samples", 'data', 'spline knots', 'spline degree', 'seed_info')
      results
      },
      error=function(cond){
        results = list(cond, data, knots ,deg, seed_info)
        names(results) = c('error', 'data', 'spline knots', 'spline degree', 'seed_info')
        return(results)
      })

    return(results)
    
  }
  
