simulate_for_study_2 = function(tau_true, rho_true, group_sizes, deg,burnin, niter, check_convg , sample_effective, neff, print_  = F){
  #generate data, whole numbers would mean that the endpoints are integers
  K             = 3
  data          = generate_data_stent(tau_true, rho_true, group_sizes, K, p =5)
  
  L           = data[[1]]        # left interval endpoints
  R           = data[[2]]        # right interval endpoints
  X           = data[[3]]        # covariate values
  group_sizes = data[[4]]
  beta_true   = data[[5]]
  alpha_true  = data[[6]]
  
  #determine measurement times (regular event-time interval endpoints) and spline knots
  times       = sort(unique(append(L,R)))
  meas_times  = times[abs(times)<Inf & times >0]
  knots       = seq(min(meas_times)*0.99, max(meas_times)*1.01, length.out = 20)

  #try to find the parameters
  results = tryCatch(
    {#perform initialization procedure
      MLE = get_MLE(L,R,X,knots, deg)
      
      # obtain MH output
      seed = round(runif(1)*10^8)
     seed_info = list(seed = seed)
      MH_samples = Gibbs_sampler_study_2(K, L, R, group_sizes, knots, deg, niter, neff , seed_info= seed_info, X=X,
                                 true_params = list(beta_true = beta_true, tau_true = tau_true, rho_true = rho_true),
                                 sample_effective = sample_effective,  init_mean = MLE,test_convg = check_convg,
                                 burnin = burnin, M =1000, a_rho = 0.001, b_rho = 0.001,
                              a_tau = 0.001, b_tau = 0.001,print_progress = print_)
      results = list(MH_samples, data, knots, deg, seed_info)
      names(results) = c("MCMC samples", 'data', 'spline knots', 'spline degree', 'seed_info')
      results
    },
    error=function(cond){
      results = list(cond, data, knots ,deg, seed_info)
      names(results) = c('error', 'data', 'spline knots', 'spline degree', 'seed_info')
    })
  return(results)
}
