rm(list=ls())
library(pracma)
library(bfp)
library(dplyr)
library(tibble)
library(splines2)
source('utils.R')
library(dplyr)
library(ggplot2)
library(tidyr)
library(Cairo)

#load data and store paths for writing output
scen = ''#
load(paste0('stent data inspection/MCMC_results_stentdata_1yr_clean',scen,'.Rds'))
results_path = ''
fig_path = paste0('../figures/')

#read the knots, degree, group_sizes from the simresults
if(exists('simresult')){
  MCMC_res = simresult$`MCMC samples`
  data = simresult$data
  knots = simresult$`spline knots`
  deg   = simresult$`spline degree`
  group_sizes = data$group_sizes
}else{
  MCMC_res = simresults$MCMC_results
  data = simresults$data
  knots = data$knots
  deg   = data$deg
  group_sizes = data$group_sizes
}

# obtaining spline matrix given the knots, measurement times and degrees
L = data$L
R = data$R
if(is.null(data$X)||Rank(data$X)<dim(data$X)[2]){p=0}else{p=dim(data$X)[[2]]}

#make histogram of sampled covariance parameters and save
samp_covs = as_tibble(MCMC_res[,seq(p+1,p+2)]) %>% rename(tau = tau_hat, rho = rho_hat)
samp_covs = samp_covs%>% pivot_longer(c(tau, rho))
samp_covs$median = median(MCMC_res[,p+1])*(samp_covs$name == 'tau') + median(MCMC_res[,p+2])*(samp_covs$name == 'rho')
CI95tau = empiricalHpd(MCMC_res[,p+1], 0.95)
CI95rho = empiricalHpd(MCMC_res[,p+2], 0.95)
samp_covs$lb = CI95tau[1]*(samp_covs$name == 'tau') + CI95rho[1]*(samp_covs$name == 'rho')
samp_covs$ub = CI95tau[2]*(samp_covs$name == 'tau') + CI95rho[2]*(samp_covs$name == 'rho')
samp_covs$name = factor(samp_covs$name, labels = c("intra~subject~covariance ~ tau[1]", "intra~treatment~covariance ~tau[2]" ))

# obtain plot parameters
if(scen == ''){
  xlim2 = c(-0.002185123,  0.013146455)
  bw = 0.0005
  ylim2 = c(0,1100)
  xlim1 = c( 0, 1.7)
  ylim1 = c(0,2.7)
  ylim_bl = c(0, 0.037)
}else{
  xlim2 = c(-0.1,  0.3)
  bw = 0.01
  ylim2 = c(0, 100)
  xlim1 = c( 0.05, 1.5)
  ylim1 = c(0,2.6)
  ylim_bl = c(0, 0.25)
}
plotdf_2 = samp_covs %>% filter(name == 'intra~treatment~covariance ~tau[2]') %>% filter((value<xlim2[2])&(value>xlim2[1])) %>% drop_na()

# make histogram of tau_2
histogram2 = plotdf_2 %>% ggplot(aes(value))+ geom_histogram(aes(y = stat(density)), col = 'black', binwidth = bw, fill = "#565656")+theme_bw( )+ylim(ylim2) +
  xlab('')+ylab('')   +
    geom_vline(aes(xintercept = median), linetype='dashed', size = 0.8)+
  theme(axis.text.y = element_blank(), axis.ticks.y = element_blank(),legend.position="none")+
    geom_vline(aes(xintercept = lb), linetype='dotdash', col = 'black' , size = 0.8)+
    geom_vline(aes(xintercept = ub), linetype='dotdash', col = 'black', size = 0.8)

ggsave(histogram2, file = paste0(results_path, fig_path,'histograms_reallife_tau2',scen,'.pdf'), device = cairo_pdf,  width = 80, height = 80, units = 'mm')

# make histogram of tau_1
plotdf_1 = samp_covs %>% filter(name == 'intra~subject~covariance ~ tau[1]') %>% drop_na()
histogram1 = plotdf_1 %>% ggplot(aes(value))+ geom_density(alpha = 0.5, col = 'black', lwd = 1, fill = "#565656")+theme_bw( ) +
  xlab('')+ylab('')  + theme(axis.text.y = element_blank(), axis.ticks.y = element_blank(),legend.position="none")+xlim(xlim1) + ylim(ylim1)+
    geom_vline(aes(xintercept = median), linetype='dashed', size = 0.8)+
    geom_vline(aes(xintercept = lb), linetype='dotdash', col = 'black' , size = 0.8)+
    geom_vline(aes(xintercept = ub), linetype='dotdash', col = 'black', size = 0.8)
ggsave(histogram1, file = paste0(results_path, fig_path,'histograms_reallife_tau1',scen,'.pdf'), device = cairo_pdf,  width = 80, height = 80, units = 'mm')

x2 = load_obj(paste0('stent data inspection/MCMC_results_stentdata_1yr_clean_RE',scen,'.Rds'))
MCMC_res2 = x2$MCMC_results


#make histogram of sampled covariance parameters and save
samp_covs2 = as_tibble(MCMC_res2[,seq(p+1,p+2)]) %>% rename(tau = tau_hat, rho = rho_hat)
samp_covs2 = samp_covs2%>% pivot_longer(c(tau, rho))
samp_covs2$median = median(MCMC_res2[,p+1])*(samp_covs2$name == 'tau') + median(MCMC_res2[,p+2])*(samp_covs2$name == 'rho')
CI95tau = empiricalHpd(MCMC_res2[,p+1], 0.95)
CI95rho = empiricalHpd(MCMC_res2[,p+2], 0.95)
samp_covs2$lb = CI95tau[1]*(samp_covs2$name == 'tau') + CI95rho[1]*(samp_covs2$name == 'rho')
samp_covs2$ub = CI95tau[2]*(samp_covs2$name == 'tau') + CI95rho[2]*(samp_covs2$name == 'rho')
samp_covs2$name = factor(samp_covs2$name, labels = c("intra~subject~covariance ~ tau[1]", "intra~treatment~covariance ~tau[2]" ))
plotdf_22 = samp_covs2 %>% filter(name == 'intra~treatment~covariance ~tau[2]') %>% filter((value<xlim2[2])&(value>xlim2[1])) %>% drop_na()

totdf = plotdf_2 %>% add_column(Sampler = 'BCSM') %>% full_join(plotdf_22 %>% add_column(Sampler = 'RE'))
# make histogram of tau_2 for both LTBCSM and RE LTM 
histogram2_RE = totdf %>% ggplot(aes(value, fill = Sampler))+ geom_histogram(aes(y= stat(density),col = Sampler), position = 'dodge', binwidth = bw)+theme_bw( )+ylim(ylim2) +
  xlab('')+ylab('')  +scale_fill_manual(values = c("BCSM" ="#565656" ,"RE" = "lightgrey"))+scale_color_manual(values = c("BCSM" ="black" ,"RE" = "black"))+
  theme(axis.text.y = element_blank(), axis.ticks.y = element_blank(),legend.position="none")
ggsave(histogram2_RE, file = paste0(results_path, fig_path,'histograms_reallife_tau2_RE',scen,'.pdf'), device = cairo_pdf,  width = 80, height = 80, units = 'mm')

# make histogram of tau_1 for both LTBCSM and RE LTM 
plotdf_12 = samp_covs2 %>% filter(name == 'intra~subject~covariance ~ tau[1]') %>% drop_na()
totdf = plotdf_1 %>% add_column(Sampler = 'BCSM') %>% full_join(plotdf_12 %>% add_column(Sampler = 'RE'))
# totdf$Sampler = factor(totdf$Sampler,levels=c( 'BCSM',"Random Effects"))
totdf$order = c(rep(2, dim(plotdf_1)[1]), rep(1, dim(plotdf_12)[1]))
histogram1_RE = totdf %>% ggplot(aes(value))+ geom_density(aes(fill = reorder(Sampler, order), linetype =  reorder(Sampler, order)), alpha = 0.5, col = 'black', lwd = 1)+theme_bw( ) +
  xlab('')+ylab('')  + theme(axis.text.y = element_blank(), axis.ticks.y = element_blank(),legend.position="none")+xlim(xlim1) + ylim(ylim1)+
  scale_fill_manual(values = c("BCSM" = "#565656","RE" = "lightgrey")) + scale_linetype_manual(values = c("BCSM" = 'solid',"RE" = "dashed"))
ggsave(histogram1_RE, file = paste0(results_path, fig_path,'histograms_reallife_tau1_RE',scen,'.pdf'), device = cairo_pdf,  width = 80, height = 80, units = 'mm')

# make baseline plots for LTBCSM  and RE BCSM
deg            = x2$data$deg
knots            = x2$data$knots
MCMC_results2= MCMC_res
MCMC_results= MCMC_res2
# obtaining spline matrix given the knots, measurement times and degrees
L = x2$data$L
R = x2$data$R
if(is.null(x2$data$X)){p=0}else{p=dim(data$X)[[2]]}
times       = sort(unique(append(L,R)))
meas_times  = times[abs(times)<Inf & times >0]
S = get_spline_matrix_points(meas_times, knots = knots, deg = deg)

#find the posterior sampled baselines and pointwise CI's
p=0
sampled_baselines = t(S%*%t(MCMC_results[,seq(p+2+1, dim(MCMC_results)[2])]))
sampled_survivalfns = pnorm(sampled_baselines/sqrt((1+MCMC_results[,'tau_hat'] + MCMC_results[,'rho_hat'])))
median_baseline = apply(sampled_survivalfns,2,median)
upperq_baseline = apply(sampled_survivalfns,2,function(x){quantile(x,0.99)[[1]]})
lowerq_baseline = apply(sampled_survivalfns,2,function(x){quantile(x,0.01)[[1]]})

sampled_baselines2 = t(S%*%t(MCMC_results2[,seq(p+2+1, dim(MCMC_results2)[2])]))
sampled_survivalfns2 = pnorm(sampled_baselines2/sqrt((1+MCMC_results2[,'tau_hat'] + MCMC_results2[,'rho_hat'])))
median_baseline2 = apply(sampled_survivalfns2,2,median)
upperq_baseline2 = apply(sampled_survivalfns2,2,function(x){quantile(x,0.99)[[1]]})
lowerq_baseline2 = apply(sampled_survivalfns2,2,function(x){quantile(x,0.01)[[1]]})

#collect in table and plot, store plot
tbl_baselines = tibble(t = c(0,meas_times), median = c(0,median_baseline2), ub = c(0,upperq_baseline2), lb = c(0,lowerq_baseline2))

baseline_plot = ggplot(data =tbl_baselines, aes(x=t)) + theme_bw()+
  scale_y_continuous(labels = scales::percent) + geom_ribbon(aes(ymin = lb, ymax = ub),alpha = 0.6, fill ="#565656", linetype = 'solid',  col = 'black')+
  xlab('time after intervention (days)') + ylab('common marginal incidence rate')+ scale_x_continuous(breaks = seq(0,365,30))+ theme(legend.position="none")+
  geom_line(aes(y = median), col = 'black',  size = 1.2)+ylim(ylim_bl)

ggsave(baseline_plot, file = paste0(results_path, fig_path, 'baseline_plot_stentstudy',scen,'.pdf'), device = cairo_pdf, width = 170, height = 110, units = 'mm')


#collect in table and plot, store plot
tbl_baselines = tibble(t = c(0,meas_times), median = c(0,median_baseline2), ub = c(0,upperq_baseline2), lb = c(0,lowerq_baseline2)) %>% add_column(Sampler = 'BCSM')
tbl_baselines_RE = tibble(t = c(0,meas_times), median = c(0,median_baseline), ub = c(0,upperq_baseline), lb = c(0,lowerq_baseline))%>% add_column(Sampler = 'Random Effects')
plottbl = tbl_baselines %>% full_join( tbl_baselines_RE) 

plottbl$order = c(rep(2, dim(plottbl)[1]/2), rep(1, dim(plottbl)[1]/2))
baseline_plot_RE = ggplot(data =plottbl, aes(x=t)) + theme_bw()+
  scale_y_continuous(labels = scales::percent) + geom_ribbon(aes(ymin = lb, ymax = ub, fill = reorder(Sampler, order), linetype = Sampler, alpha = Sampler), col = 'black')+
  xlab('time after intervention (days)') + ylab('common marginal incidence rate')+ scale_x_continuous(breaks = seq(0,365,30))+ theme(legend.position="none")+
   scale_fill_manual(values = c('BCSM'  = "#565656","Random Effects" = "lightgrey"))+ylim(ylim_bl)+
   scale_alpha_manual(values = c('BCSM' =0.6, 'Random Effects' = 0.6))+ geom_line(aes(y = median, linetype = Sampler), col = 'black',  size = 1.2)
 
ggsave(baseline_plot_RE, file = paste0(results_path, fig_path, 'baseline_plot_stentstudy_RE',scen,'.pdf'), device = cairo_pdf, width = 170, height = 110, units = 'mm')





