rm(list=ls())
library(pracma)
library(bfp)
library(dplyr)
library(tibble)
library(splines2)
source('utils.R')
library(dplyr)
library(ggplot2)
library(tidyr)
library(Cairo)
library(invgamma)
library(latex2exp)
library(coda)

results_path = ''
fig_path = paste0('figures/')
save_ = T
app = '_both'
scens = c('','_SR0_01', '_SR0_1', '_SR1_0')
scens = sapply(scens, function(x){paste0(x, app)})

load(paste0('stent data inspection/MCMC_results_stentdata_1yr_clean',scens[1],'.Rds'))
simresults =x
data = simresults$data
group_sizes = data$group_sizes
scens2=c( 0.001,0.01,0.1,1.0)
pp =c()

#cacuat prior probs
M =1000000
for(scen in scens2){
  l1 = rinvgamma(M, shape =0.001, rate=0.001)
  if(any(is.infinite(l1)))
    l1[is.infinite(l1)] = rinvgamma(sum(is.infinite(l1)), shape =0.001, rate=0.001)
  pp = c(pp, mean(pinvgamma(l1,  shape = scen, rate=scen)))
}
pp

# combine results with different prior parameters
res= c()
for(scen in scens){
  row_res  = c()
  
  #load data and store paths for writing output
  load(paste0('stent data inspection/MCMC_results_stentdata_1yr_clean',scen,'.Rds'))
  simresults =x
  data = simresults$data
  group_sizes = data$group_sizes

  #read data

  MCMC_res = simresults$MCMC_results
  data = simresults$data
  knots = data$knots
  deg   = data$deg
  group_sizes = data$group_sizes
  
  row_res  = c(row_res, mean(MCMC_res[,1]<=0))
  row_res  = c(row_res, median(MCMC_res[,2]))
  hpd = empiricalHpd(MCMC_res[,2],0.95)
  row_res  = c(row_res,hpd[1])
  row_res  = c(row_res,hpd[2])
  
  x2 = load_obj(paste0('stent data inspection/MCMC_results_stentdata_1yr_clean_RE',scen,'.Rds'))
  MCMC_res2 = x2$MCMC_results
  # assertthat::assert_that(min(effectiveSize(MCMC_res2))>=600)
  # assertthat::assert_that(min(effectiveSize(MCMC_res))>=600)
  
  row_res  = c(row_res, median(MCMC_res2[,2]))
  hpd = empiricalHpd(MCMC_res2[,2],0.95)
  row_res  = c(row_res,hpd[1])
  row_res  = c(row_res,hpd[2])
  
  deg            = x2$data$deg
  knots            = x2$data$knots
  MCMC_results2= MCMC_res
  MCMC_results= MCMC_res2
  # obtaining spline matrix given the knots, measurement times and degrees
  L = x2$data$L
  R = x2$data$R
  if(is.null(x2$data$X)){p=0}else{p=dim(data$X)[[2]]}
  times       = sort(unique(append(L,R)))
  meas_times  = seq(10,360,10)#times[abs(times)<Inf & times >0]
  S = get_spline_matrix_points(meas_times, knots = knots, deg = deg)
  
  #find the posterior sampled baselines and pointwise CI's
  p=0
  sampled_baselines = t(S%*%t(MCMC_results[,seq(p+2+1, dim(MCMC_results)[2])]))
  sampled_survivalfns = pnorm(sampled_baselines/sqrt((1+MCMC_results[,'tau_hat'] + MCMC_results[,'rho_hat'])))
  median_baseline = apply(sampled_survivalfns,2,median)
  upperq_baseline = apply(sampled_survivalfns,2,function(x){quantile(x,0.99)[[1]]})
  lowerq_baseline = apply(sampled_survivalfns,2,function(x){quantile(x,0.01)[[1]]})
  
  sampled_baselines2 = t(S%*%t(MCMC_results2[,seq(p+2+1, dim(MCMC_results2)[2])]))
  sampled_survivalfns2 = pnorm(sampled_baselines2/sqrt((1+MCMC_results2[,'tau_hat'] + MCMC_results2[,'rho_hat'])))
  median_baseline2 = apply(sampled_survivalfns2,2,median)
  upperq_baseline2 = apply(sampled_survivalfns2,2,function(x){quantile(x,0.99)[[1]]})
  lowerq_baseline2 = apply(sampled_survivalfns2,2,function(x){quantile(x,0.01)[[1]]})
  
  row_res  = c(row_res,mean(abs(median_baseline2)))
  row_res  = c(row_res,mean(upperq_baseline2 ))
  row_res  = c(row_res,mean(lowerq_baseline2 ))
  row_res  = c(row_res,mean(abs(median_baseline)))
  row_res  = c(row_res,mean(upperq_baseline))
  row_res  = c(row_res,mean(lowerq_baseline))

  
  row_res  = c(row_res, median(MCMC_res[,1]))
  hpd = empiricalHpd(MCMC_res[,1],0.95)
  row_res  = c(row_res,hpd[1])
  row_res  = c(row_res,hpd[2])
  row_res  = c(row_res, median(MCMC_res2[,1]))
  hpd = empiricalHpd(MCMC_res2[,1],0.95)
  row_res  = c(row_res,hpd[1])
  row_res  = c(row_res,hpd[2])
  
  
  res = rbind(res,row_res)
}

# plot prior prob tau_2<=0 vs log_10(c)
res = as_tibble(res) %>% add_column('c' = c(0.001,0.01,0.1,1.0), .before = 1)
colnames(res)[1]='c'
colnames(res)[-1] = sapply(seq(dim(res)[2]-1), function(i){paste0('V',i)})
prob_plot_priorrobust =ggplot(res, aes(x = log(c)/log(10))) + geom_line(aes(y = log((V1/pp)/((1-V1)/(1-pp))))) + ylab(TeX("$P(\\tau_2\\leq 0)$"))+ xlab(TeX("$\\log_{10}(c)$"))+theme_bw(base_size = 15)+geom_point(aes(y=log((V1/pp)/((1-V1)/(1-pp)))))
prob_plot_priorrobust =ggplot(res, aes(x = log(c)/log(10))) + geom_line(aes(y = V1)) + ylab(TeX("$Pr(\\tau_2\\leq 0 \\,|\\,  \\textbf{L,R})$"))+ xlab(TeX("$\\log_{10}(c)$"))+theme_bw(base_size = 15)+geom_point(aes(y=V1))
if(save_){
ggsave(prob_plot_priorrobust, file = paste0(results_path, fig_path, 'prob_plot_priorrobust',app,'.pdf'), device = cairo_pdf, width = 160, height = 160, units = 'mm')
}

# plot statistics tau_1 vs log_10(c)
res_tau1= tibble(c = c(res$c,res$c), median=c(res$V2, res$V5), lb = c(res$V3, res$V6), ub =  c(res$V4, res$V7), name = c(rep('LT-BCSM', 4),rep('RE LTM', 4)))
tau1_plot_priorrobust=ggplot(res_tau1, aes(x = log(c)/log(10), color= name, markerstyle = name, linetype = name))+ ylab(TeX("$\\tau_1$"))+ xlab(TeX("$\\log_{10}(c)$"))+theme_bw(base_size = 15)+ 
  geom_line(aes(y = median), lwd=1.25)+geom_point(aes(y= median))+geom_ribbon(aes(ymin = lb, ymax = ub), alpha = 0.2,lwd=1)+ theme(legend.position = "none")+
  scale_fill_manual(values = c('LT-BCSM'  = "#565656",'RE LTM' = "lightgrey"))+
  scale_color_manual(values = c('LT-BCSM'  = "black",'RE LTM' = "black"))
  scale_linetype_manual(values = c('LT-BCSM'  = "solid",'RE LTM' = "dashed"))
if(save_){
  ggsave(tau1_plot_priorrobust, file = paste0(results_path, fig_path, 'tau1_plot_priorrobust',app,'.pdf'), device = cairo_pdf, width = 160, height = 160, units = 'mm')
}

# plot statistics bl vs log_10(c)
res_bl= tibble(c = c(res$c,res$c), median=c(res$V8, res$V11), lb = c(res$V9, res$V12), ub =  c(res$V10, res$V13), name = c(rep('LT-BCSM', 4),rep('RE LTM', 4)))
bl_plot_priorrobust=ggplot(res_bl, aes(x = log(c)/log(10), color= name, markerstyle = name, linetype = name))+ ylab(TeX("incidence rate"))+ xlab(TeX("$\\log_{10}(c)$"))+theme_bw(base_size = 15)+ 
    geom_line(aes(y = median), lwd=1.25)+geom_point(aes(y= median))+geom_ribbon(aes(ymin = lb, ymax = ub), alpha = 0.2,lwd=1)+ theme(legend.position = "none")+
    scale_fill_manual(values = c('LT-BCSM'  = "#565656",'RE LTM' = "lightgrey"))+
    scale_color_manual(values = c('LT-BCSM'  = "black",'RE LTM' = "black"))
  scale_linetype_manual(values = c('LT-BCSM'  = "solid",'RE LTM' = "dashed"))
  if(save_){
    ggsave(bl_plot_priorrobust, file = paste0(results_path, fig_path, 'bl_plot_priorrobust',app,'.pdf'), device = cairo_pdf, width = 160, height = 160, units = 'mm')
  }
  
  
  # plot statistics tau_2 vs log_10(c)
  res_tau2= tibble(c = c(res$c,res$c), median=c(res$V14, res$V17), lb = c(res$V15, res$V18), ub =  c(res$V16, res$V19), name = c(rep('LT-BCSM', 4),rep('RE LTM', 4)))
  tau2_plot_priorrobust=ggplot(res_tau2, aes(x = log(c)/log(10), color= name, markerstyle = name, linetype = name))+ ylab(TeX("$\\tau_2$"))+ xlab(TeX("$\\log_{10}(c)$"))+theme_bw(base_size = 15)+ 
    geom_line(aes(y = median), lwd=1.25)+geom_point(aes(y= median))+geom_ribbon(aes(ymin = lb, ymax = ub), alpha = 0.2,lwd=1)+ theme(legend.position = "none")+
    scale_fill_manual(values = c('LT-BCSM'  = "#565656",'RE LTM' = "lightgrey"))+
    scale_color_manual(values = c('LT-BCSM'  = "black",'RE LTM' = "black"))
  scale_linetype_manual(values = c('LT-BCSM'  = "solid",'RE LTM' = "dashed"))
  if(save_){
    ggsave(tau2_plot_priorrobust, file = paste0(results_path, fig_path, 'tau2_plot_priorrobust',app,'.pdf'), device = cairo_pdf, width = 160, height = 160, units = 'mm')
  }
 
