library('pracma')
library('rstudioapi')
library('Matrix')
library('haven')
library('dplyr')
library('tibble')
library(splines2)
rm(list=ls())


#import functions
source('utils.R')
source('get_MLE.R')
source("Gibbs_sampler_realistic.R")
source("Gibbs_sampler_RE.R")
source("parameter sampling/sample_Z.R")
source("parameter sampling/sample_covariances.R")
source("parameter sampling/sample_beta.R")
source("parameter sampling/sample_gamma.R")

# reading/transforming data to correct format
data = as_tibble(read_sav('stent data inspection/BIORESORT.sav'))
data = data %>% arrange(Randomized_stent)
#obtaining group sizes
group_sizes = table(data$Randomized_stent)

#three event-types
surv1 = data$Days_to_cardiac_death_or_consentwithdrawal_1yr
surv2 = data$Days_to_TVMI_1yr
surv3 =data$Days_to_TV_revasc_PCIandCABG_1yr

# assert that the minimum over event times included is indeed target vessel failure
TVF_approx = sapply(seq(length(surv1)), function(i){min(surv1[i], surv2[i], surv3[i])})
assertthat::assert_that(all(TVF_approx == data$Days_to_TVF_1yr))

#obtaining L (with possible left censoring)
R = matrix(rbind(surv1, surv2, surv3), ncol = 1)
surv1_L = surv1
surv1_L[(data$Death_any_1yr==1) &!(data$Death_Cardiac_1yr==1)]=1# left cens in case of non-cardiac death
surv2_L = surv2
surv2_L[data$Death_any_1yr==1]=1# left cens in case of any death
surv3_L = surv3
surv3_L[data$Death_any_1yr==1]=1 # left cens in case of any death
R = matrix(rbind(surv1_L, surv2_L, surv3_L), ncol = 1)
L = R - 1
L[L<=0] = 0


#obtaining R (with right-censoring)
surv1[data$Death_Cardiac_1yr==0]= Inf
surv2[data$TVMI_1yr==0]= Inf
surv3[data$TV_revasc_PCIandCABG_1yr==0 ]= Inf 
R = matrix(rbind(surv1, surv2, surv3), ncol = 1)

#MCMC params
K = 3
deg            = 4
niter          = 6000  # length of Markov chain
burnin         = 3000    # burn-in period Markov chain

#determine measurement times (regular event-time interval endpoints) and spline knots
times       = sort(unique(append(L,R)))
meas_times  = times[abs(times)<Inf & times >0]
knots       = seq(min(meas_times)*0.99, max(meas_times)*1.01, length.out = 20)

M  = 1000 # C in paper, prior support for spline hierarchical param


neff = 600
test_convg = T
seed_info = list(seed = as.integer(Sys.time()), random_seed = .Random.seed)
previous_results = NULL


#perform initialization procedure with MLE
MLE = get_MLE(L,R,X=NULL,knots, deg)

name = 'LTM_SRnormal_RE'
simresults = Gibbs_sampler_RE(K, L, R, group_sizes, knots, deg, niter, neff, burnin = burnin , test_convg = test_convg,
                                     print_progress = T, write_ = T, sample_effective = T, save_intermediate = T, init_mean = MLE, a_tau = 0.001, b_tau = 0.001,
                                     a_rho = 0.001, b_rho = 0.001, 
                                     seed_info = seed_info, previous_results = previous_results, name = name, M = 1000)

save(simresults, file = paste0('MCMC_results_stentdata_1yr_clean_RE.Rds'))
